from django.core.exceptions import ObjectDoesNotExist, ValidationError
from django.utils.translation import gettext_lazy as _
from pulp_ansible.app.models import AnsibleDistribution
from pulpcore.plugin.models import SigningService
from rest_framework import status
from rest_framework.response import Response

from galaxy_ng.app.access_control import access_policy
from galaxy_ng.app.api import base as api_base
from galaxy_ng.app.tasks import call_sign_task


class CollectionSignView(api_base.APIView):
    action = "sign"
    permission_classes = [access_policy.CollectionAccessPolicy]

    def post(self, request, *args, **kwargs):
        """Creates a signature for the content units specified in the request.


        Base URL: _ui/v1/collection_signing/

        Optional URLS:
        - _ui/v1/collection_signing/<distro_base_path>/
        - _ui/v1/collection_signing/<distro_base_path>/<namespace>/
        - _ui/v1/collection_signing/<distro_base_path>/<namespace>/<collection>/
        - _ui/v1/collection_signing/<distro_base_path>/<namespace>/<collection>/<version>/

        The request body should contain a JSON object with the following keys:

        Required
            - signing_service: The name of the signing service to use
            - distro_base_path: The name of the distro_base_path to add the signatures
              (if not provided via URL)

        Optional
            - content_units: A list of content units UUIDS to be signed.
            (if content_units is ["*"], all units under the repo will be signed)
            OR
            - namespace: Namespace name (if not provided via URL)
            (if only namespace is specified, all collections under that namespace will be signed)

        Optional (one or more)
            - collection: Collection name (if not provided via URL)
            (if collection name is added, all versions under that collection will be signed)
            - version: The version of the collection to sign (if not provided via URL)
            (if version is specified, only that version will be signed)
        """

        self.kwargs = kwargs
        signing_service = self._get_signing_service(request)
        repository = self.get_repository(request)
        content_units = self._get_content_units_to_sign(request, repository)

        sign_task = call_sign_task(signing_service, repository, content_units)
        return Response(data={"task_id": sign_task.pk}, status=status.HTTP_202_ACCEPTED)

    def _get_content_units_to_sign(self, request, repository):
        """Returns a list of pks for content units to sign.

        If `content_units` is specified in the request, it will be used.
        Otherwise, will use the filtering options specified in the request.
        namespace, collection, version can be used to filter the content units.
        """
        if request.data.get("content_units"):
            return request.data["content_units"]
        else:
            try:
                namespace = self.kwargs.get("namespace") or request.data["namespace"]
            except KeyError:
                raise ValidationError(_("Missing required field: namespace"))

            query_params = {
                "pulp_type": "ansible.collection_version",
                "ansible_collectionversion__namespace": namespace,
            }

            collection = self.kwargs.get("collection") or request.data.get("collection")
            version = self.kwargs.get("version") or request.data.get("version")

            if collection:
                query_params["ansible_collectionversion__name"] = collection
            if version:
                query_params["ansible_collectionversion__version"] = version

            content_units = repository.content.filter(**query_params).values_list("pk", flat=True)
            if not content_units:
                raise ValidationError(_("No content units found for: %s") % query_params)

            return [str(item) for item in content_units]

    def get_repository(self, request):
        """Retrieves the repository object from the request distro_base_path.

        :param request: the request object
        :return: the repository object

        NOTE: This method is used by the access policies.
        """
        try:
            distro_name = self.kwargs.get("path") or request.data["distro_base_path"]
        except KeyError:
            raise ValidationError(_("distro_base_path field is required."))

        try:
            return AnsibleDistribution.objects.get(base_path=distro_name).repository
        except ObjectDoesNotExist:
            raise ValidationError(_("Distribution %s does not exist.") % distro_name)

    def _get_signing_service(self, request):
        try:
            return SigningService.objects.get(name=request.data["signing_service"])
        except KeyError:
            raise ValidationError(_("signing_service field is required."))
        except ObjectDoesNotExist:
            raise ValidationError(
                _('Signing service "%s" does not exist.') % request.data["signing_service"]
            )



import logging
import mimetypes

from rest_framework import serializers
from rest_framework.exceptions import ValidationError, _get_error_details

from galaxy_ng.app.api.ui.serializers.base import Serializer
from galaxy_ng.app.api.utils import parse_collection_filename

log = logging.getLogger(__name__)


class CollectionUploadSerializer(Serializer):
    """
    A serializer for the Collection One Shot Upload API.
    """

    file = serializers.FileField(required=True)

    sha256 = serializers.CharField(required=False, default=None)

    class Meta():
        ref_name = "CollectionUploadWithDownloadUrlSerializer"

    def to_internal_value(self, data):
        """Parse and validate collection filename."""
        data = super().to_internal_value(data)

        errors = {}

        filename = data["file"].name

        try:
            filename_tuple = parse_collection_filename(filename)
        except ValueError as exc:
            errors['filename'] = _get_error_details(exc, default_code='invalid')
            log.error('CollectionUploadSerializer validation of filename failed: %s',
                      errors['filename'])
            raise ValidationError(errors)

        data.update({
            "filename": filename_tuple,
            "expected_namespace": filename_tuple.namespace,
            "expected_name": filename_tuple.name,
            "expected_version": filename_tuple.version,
            "mimetype": (mimetypes.guess_type(filename)[0] or 'application/octet-stream')
        })
        return data



from collections import OrderedDict

from django.conf import settings

from rest_framework import (
    serializers,
    viewsets,
    views,
    permissions,
    generics
)

from rest_framework.settings import perform_import


GALAXY_EXCEPTION_HANDLER = perform_import(
    settings.GALAXY_EXCEPTION_HANDLER,
    'GALAXY_EXCEPTION_HANDLER'
)
GALAXY_AUTHENTICATION_CLASSES = perform_import(
    settings.GALAXY_AUTHENTICATION_CLASSES,
    'GALAXY_AUTHENTICATION_CLASSES'
)
GALAXY_PAGINATION_CLASS = perform_import(
    settings.GALAXY_PAGINATION_CLASS,
    'GALAXY_PAGINATION_CLASS'
)


class _MustImplementPermission(permissions.BasePermission):
    def has_permission(self, request, view):
        raise NotImplementedError("subclass must implement permission_classes")


class LocalSettingsMixin:
    authentication_classes = GALAXY_AUTHENTICATION_CLASSES
    pagination_class = GALAXY_PAGINATION_CLASS
    permission_classes = [_MustImplementPermission]

    def get_exception_handler(self):
        return GALAXY_EXCEPTION_HANDLER


class APIView(LocalSettingsMixin, views.APIView):
    pass


class ViewSet(LocalSettingsMixin, viewsets.ViewSet):
    pass


class GenericAPIView(LocalSettingsMixin, generics.GenericAPIView):
    pass


class GenericViewSet(LocalSettingsMixin, viewsets.GenericViewSet):
    pass


class ModelViewSet(LocalSettingsMixin, viewsets.ModelViewSet):
    pass


class RelatedFieldsBaseSerializer(serializers.Serializer):
    """
    Serializer only returns fields specified in 'include_related' query param.

    This allows for fields that require more database queries to be optionally
    included in API responses, which lowers the load on the backend. This is
    intended as a way to include extra data in list views.

    Usage:

    This functions the same as DRF's base `serializers.Serializer` class with the
    exception that it will only return fields specified in the `?include_related=`
    query parameter.

    Example:

    MySerializer(RelatedFieldsBaseSerializer):
        foo = CharField()
        bar = CharField()

    MySerializer will return:

    {"foo": None} when called with `?include_related=foo` and {"foo": None, "bar" None}
    when called with `?include_related=foo&include_related=bar`.
    """

    def __init__(self, *args, **kwargs):
        # This should only be included as a sub serializer and shouldn't be used for
        # updating objects, so set read_only to true
        kwargs['read_only'] = True
        return super().__init__(*args, **kwargs)

    def to_representation(self, instance):
        result = OrderedDict()
        fields = self._readable_fields
        request = self.context.get('request', None)
        if request:

            # TODO: Figure out how to make `include_related` show up in the open API spec
            include_fields = request.GET.getlist('include_related')

            if len(include_fields) > 0:
                for field in fields:
                    if field.field_name in include_fields:
                        result[field.field_name] = field.to_representation(instance)
        else:
            # When there is no request present, it usually means that the serializer is
            # being inspected by DRF spectacular to generate the open API spec. In that
            # case, this should act like a normal serializer.
            return super().to_representation(instance)

        return result



"""test_package.py - Tests related to setup.py

See: https://issues.redhat.com/browse/AAH-1545

"""

import pytest
import subprocess
import tempfile


pytestmark = pytest.mark.qa  # noqa: F821


@pytest.mark.standalone_only
@pytest.mark.package
@pytest.mark.parametrize(
    "env_vars",
    [
        {},
        {'LOCK_REQUIREMENTS': '0'}
    ]
)
def test_package_install(env_vars):
    """smoktest setup.py"""

    with tempfile.TemporaryDirectory(prefix='galaxy_ng_testing_') as basedir:

        # make a venv
        pid = subprocess.run(f'python3 -m venv {basedir}/venv', shell=True)
        assert pid.returncode == 0

        # install the package
        cmd = f'{basedir}/venv/bin/pip install .'
        if env_vars:
            for k, v in env_vars.items():
                cmd = f'{k}={v} {cmd}'
        pid = subprocess.run(cmd, shell=True)
        assert pid.returncode == 0

        # check the installed packages
        cmd = f'{basedir}/venv/bin/pip list'
        pid = subprocess.run(cmd, shell=True, stdout=subprocess.PIPE)
        assert pid.returncode == 0

        package_list = pid.stdout.decode('utf-8')
        package_list = package_list.split('\n')
        package_list = [x.strip() for x in package_list if x.strip()]
        package_list = [x for x in package_list if not x.startswith('Package')]
        package_list = [x for x in package_list if not x.startswith('-')]
        package_names = [x.split()[0] for x in package_list]

        assert 'pulpcore' in package_names
        assert 'pulp-ansible' in package_names
        assert 'pulp-container' in package_names
        assert 'galaxy-ng' in package_names



"""test_landing_page.py - Test related to landing page endpoint.
"""
import pytest

from ..utils import get_client


@pytest.mark.cloud_only
def test_landing_page(ansible_config):
    """Tests whether the landing page returns the expected fields and numbers."""

    api_client = get_client(
        config=ansible_config("ansible_insights"), request_token=True, require_auth=True
    )

    resultsDict = api_client("/api/automation-hub/_ui/v1/landing-page")

    assert resultsDict["estate"]["items"][0]
    assert resultsDict["estate"]["items"][0]["shape"]["title"] == "Collections"
    assert resultsDict["estate"]["items"][0]["count"] == 0

    assert resultsDict["estate"]["items"][1]
    assert resultsDict["estate"]["items"][1]["shape"]["title"] == "Partners"
    assert resultsDict["estate"]["items"][1]["count"] == 2

    assert resultsDict["recommendations"]['recs']
    assert len(resultsDict["recommendations"]['recs']) == 1

    assert resultsDict["configTryLearn"]["configure"]
    assert len(resultsDict["configTryLearn"]["configure"]) == 1

    assert resultsDict["configTryLearn"]["try"]
    assert len(resultsDict["configTryLearn"]["try"]) == 2

    assert resultsDict["configTryLearn"]["learn"]
    assert len(resultsDict["configTryLearn"]["learn"]) == 3



import os
import shutil

import pytest
# from orionutils.generator import build_collection

from .constants import USERNAME_PUBLISHER
from .utils import ansible_galaxy, get_client, set_certification
from .utils import upload_artifact as _upload_artifact
from .utils import get_all_namespaces
from .utils import build_collection

from orionutils.utils import increment_version

MARKER_CONFIG = """
qa: Mark tests to run in the vortex job.
galaxyapi_smoke: Smoke tests for galaxy-api backend.
standalone_only: Tests that should not run against the Insights version of Hub.
cloud_only: Tests that should not run against the standalone version of Hub.
cli: Tests that shell out to the real ansible-galaxy cli.
ui: Tests that navigate the UI via selenium.
ui_standalone: UI tests that only work in standalone mode.
smoke: Same as galaxyapi_smoke?
prod_status: For checking prod?
busted: Doesn't work yet.
stage_health: For checking stage?
namespace: Tests that manipulate namespaces.
certification: Related to certification.
collection_move: Related to moving collection versions.
collection_delete: Deletes collections.
collection_version_delete: Deletes collections versions.
collection_signing: Related to collection signing.
delete: Tests for deleting objects.
move: Related to the move api.
synclist: Related to synclist object and synclist repo.
openapi: Checks the openapi schema and routes.
openapi_generate_bindings: Verifies pulp client bindings generator
package: tests for the pip packaging
"""


def pytest_configure(config):
    for line in MARKER_CONFIG.split('\n'):
        if not line:
            continue
        config.addinivalue_line('markers', line)


class AnsibleConfigFixture(dict):

    # This is "kinda" like how IQE uses dynaconf, but
    # much simpler and not such a huge tangled mess to
    # understand ...

    # The class is instantiated with a "namespace" of sorts,
    # which sets the context for the rest of the config ...
    #   config = ansible_config("ansible_partner")
    #   config = ansible_config("ansible_insights")

    def __init__(self, namespace1, namespace=None):
        self.namespace = namespace

    def __repr__(self):
        return f'<AnsibleConfigFixture: {self.namespace}>'

    def __getitem__(self, key):

        if key == 'url':
            # The "url" key is actually the full url to the api root.
            return os.environ.get(
                'HUB_API_ROOT',
                'http://localhost:8002/api/automation-hub/'
            )

        elif key == 'auth_url':
            # The auth_url value should be None for a standalone stack.
            return os.environ.get(
                'HUB_AUTH_URL',
                None
            )

        elif key == 'token':
            return os.environ.get(
                'HUB_TOKEN',
                None
            )

        elif key == 'username':
            return os.environ.get(
                'HUB_USERNAME',
                'admin'
            )

        elif key == 'password':
            return os.environ.get(
                'HUB_PASSWORD',
                'admin'
            )

        elif key == 'hub_use_inbound':
            # This value will be compared to "use_distribution" in the
            # test_api_publish test. I assume it has to do with pulp's
            # concept of "distribution" but not sure what it actually
            # means in this case
            return True

        elif key == 'use_move_endpoint':
            # tells the tests whether or not to try to mark
            # an imported collection as "published". This happens
            # automatically in the default config for standalone,
            # so should return False in that case ...

            if os.environ.get('HUB_USE_MOVE_ENDPOINT'):
                val = os.environ['HUB_USE_MOVE_ENDPOINT']
                if str(val) in ['1', 'True', 'true']:
                    return True

            # standalone ...
            return False

            # cloud ...
            # return True

        else:
            raise Exception(f'Uknown config key: {self.namespace}.{key}')

        return super().__getitem__(key)

    def get(self, key):
        return self.__getitem__(key)


class AnsibleAppFixture():
    def __init__(self):
        self.config = AnsibleConfigFixture('APP')
        setattr(
            self.config,
            'AUTOMATION_HUB',
            AnsibleConfigFixture('AUTOMATION_HUB')
        )


@pytest.fixture
def ansible_config():
    return AnsibleConfigFixture


@pytest.fixture(scope="function")
def published(ansible_config, artifact):

    # make sure the expected namespace exists ...
    api_client = get_client(ansible_config("ansible_insights"))
    existing = dict((x['name'], x) for x in get_all_namespaces(api_client=api_client))
    if artifact.namespace not in existing:
        payload = {'name': artifact.namespace, 'groups': []}
        api_client('/api/automation-hub/v3/namespaces/', args=payload, method='POST')

    # Publish and certify ...
    ansible_galaxy(
        f"collection publish {artifact.filename} -vvv --server=automation_hub",
        ansible_config=ansible_config("ansible_partner", namespace=artifact.namespace)
    )
    set_certification(api_client, artifact)

    return artifact


@pytest.fixture(scope="function")
def certifiedv2(ansible_config, artifact):
    """ Create and publish+certify collection version N and N+1 """

    # make sure the expected namespace exists ...
    api_client = get_client(ansible_config("ansible_insights"))
    existing = dict((x['name'], x) for x in get_all_namespaces(api_client=api_client))
    if artifact.namespace not in existing:
        payload = {'name': artifact.namespace, 'groups': []}
        api_client('/api/automation-hub/v3/namespaces/', args=payload, method='POST')

    # Publish and certify v1 ...
    ansible_galaxy(
        f"collection publish {artifact.filename}",
        ansible_config=ansible_config("ansible_partner", namespace=artifact.namespace)
    )
    set_certification(api_client, artifact)

    # Increase collection version
    new_version = increment_version(artifact.version)
    artifact2 = build_collection(
        key=artifact.key,
        namespace=artifact.namespace,
        name=artifact.name,
        version=new_version
    )

    # Publish and certify newer version ...
    ansible_galaxy(
        f"collection publish {artifact2.filename}",
        ansible_config=ansible_config("ansible_partner", namespace=artifact.namespace)
    )
    set_certification(api_client, artifact2)

    return (artifact, artifact2)


@pytest.fixture(scope="function")
def uncertifiedv2(ansible_config, artifact):
    """ Create and publish collection version N and N+1 but only certify N"""

    # make sure the expected namespace exists ...
    api_client = get_client(ansible_config("ansible_insights"))
    existing = dict((x['name'], x) for x in get_all_namespaces(api_client=api_client))
    if artifact.namespace not in existing:
        payload = {'name': artifact.namespace, 'groups': []}
        api_client('/api/automation-hub/v3/namespaces/', args=payload, method='POST')

    # Publish and certify v1 ...
    ansible_galaxy(
        f"collection publish {artifact.filename}",
        ansible_config=ansible_config("ansible_partner", namespace=artifact.namespace)
    )
    set_certification(api_client, artifact)

    # Increase collection version
    new_version = increment_version(artifact.version)
    artifact2 = build_collection(
        key=artifact.key,
        namespace=artifact.namespace,
        name=artifact.name,
        version=new_version
    )

    # Publish but do -NOT- certify newer version ...
    ansible_galaxy(
        f"collection publish {artifact2.filename}",
        ansible_config=ansible_config("ansible_partner", namespace=artifact.namespace)
    )

    return (artifact, artifact2)


@pytest.fixture
def app():
    return AnsibleAppFixture()


@pytest.fixture(scope="function")
def artifact():
    """Generate a randomized collection for testing."""

    artifact = build_collection(
        "skeleton",
        config={
            "namespace": USERNAME_PUBLISHER,
            "tags": ["tools", "database"],
        },
    )
    return artifact


@pytest.fixture
def upload_artifact():
    return _upload_artifact


@pytest.fixture
def cleanup_collections(request):
    """Clean created resources during test executions."""

    def cleanup():
        path = os.path.expanduser(
            f"~/.ansible/collections/ansible_collections/{USERNAME_PUBLISHER}/"
        )
        if os.path.exists(path):
            shutil.rmtree(path)

    request.addfinalizer(cleanup)



# Generated by Django 3.2.12 on 2022-03-23 18:37

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('galaxy', '0026_migrate_to_core_content_guard'),
    ]

    operations = [
        migrations.DeleteModel(
            name='ContentRedirectContentGuard',
        ),
    ]



"""test_move.py - Tests related to the move api.

See: https://issues.redhat.com/browse/AAH-1268

"""
import pytest
import time
from urllib.parse import urljoin
from orionutils.generator import build_collection

from ..constants import USERNAME_PUBLISHER
from ..utils import get_client
from ..utils import set_certification


pytestmark = pytest.mark.qa  # noqa: F821


@pytest.mark.galaxyapi_smoke
@pytest.mark.certification
@pytest.mark.collection_move
@pytest.mark.move
def test_move_collection_version(ansible_config, upload_artifact):
    """Tests whether a colleciton can be moved from repo to repo"""

    config = ansible_config("ansible_partner")
    api_client = get_client(
        config=config,
        request_token=True,
        require_auth=True
    )

    def get_all_collections():
        collections = {
            'staging': {},
            'published': {}
        }
        for repo in collections.keys():
            next_page = f'/api/automation-hub/_ui/v1/collection-versions/?repository={repo}'
            while next_page:
                resp = api_client(next_page)
                for _collection in resp['data']:
                    key = (_collection['namespace'], _collection['name'], _collection['version'])
                    collections[repo][key] = _collection
                next_page = resp.get('links', {}).get('next')
        return collections

    pre = get_all_collections()
    artifact = build_collection(
        "skeleton",
        config={
            "namespace": USERNAME_PUBLISHER,
            "tags": ["tools"],
        }
    )
    ckey = (artifact.namespace, artifact.name, artifact.version)
    assert ckey not in pre['staging']
    assert ckey not in pre['published']

    # import and wait ...
    resp = upload_artifact(config, api_client, artifact)
    ready = False
    url = urljoin(config["url"], resp["task"])
    while not ready:
        resp = api_client(url)
        ready = resp["state"] not in ("running", "waiting")
        time.sleep(1)
    assert resp['state'] == 'completed'

    # Make sure it ended up in staging but not in published ...
    before = get_all_collections()
    assert ckey in before['staging']
    assert ckey not in before['published']

    # Certify and check the response...
    cert_result = set_certification(api_client, artifact)
    assert cert_result['namespace']['name'] == artifact.namespace
    assert cert_result['name'] == artifact.name
    assert cert_result['version'] == artifact.version
    assert cert_result['href'] is not None
    assert cert_result['metadata']['tags'] == ['tools']

    # Make sure it's moved to the right place ...
    after = get_all_collections()
    assert ckey not in after['staging']
    assert ckey in after['published']

    # Make sure an error is thrown if move attempted again ...
    failed = None
    try:
        cert_result = set_certification(api_client, artifact)
        failed = False
    except Exception:
        failed = True
    assert failed



"""test_openapi.py - Tests related to open api generation and schema.

See: https://issues.redhat.com/browse/AAH-1169

"""

import json
import os
import pytest
import subprocess
import tempfile

from openapi_spec_validator import validate_spec

from ..utils import get_client


pytestmark = pytest.mark.qa  # noqa: F821


PULPY_VARIABLES = [
    '{ansible_collection_href}',
    '{ansible_collection_version_href}',
    '{ansible_collection_import_href}',
    '{role_href}',
    '{task_group_href}',
    '{upload_href}',
    '{worker_href}'
]


@pytest.mark.openapi
def test_galaxy_openapi_no_pulp_variables(ansible_config):
    """Tests whether openapi.json has valid path names"""

    config = ansible_config("ansible_partner")
    api_client = get_client(
        config=config,
        request_token=True,
        require_auth=True
    )

    galaxy_spec = api_client('/api/automation-hub/v3/openapi.json')
    assert 'paths' in galaxy_spec

    paths_keys = list(galaxy_spec['paths'].keys())
    for path in paths_keys:
        assert not path.startswith('{')


@pytest.mark.openapi
def test_galaxy_openapi_validation(ansible_config):
    """Tests whether openapi.json passes openapi linter"""

    config = ansible_config("ansible_partner")
    api_client = get_client(
        config=config,
        request_token=True,
        require_auth=True
    )

    galaxy_spec = api_client('/api/automation-hub/v3/openapi.json')
    validate_spec(galaxy_spec)


@pytest.mark.standalone_only
@pytest.mark.openapi
def test_pulp_openapi_has_variables(ansible_config):
    """Tests whether openapi.json has valid path names for pulp"""

    config = ansible_config("ansible_partner")
    api_client = get_client(
        config=config,
        request_token=True,
        require_auth=True
    )

    pulp_spec = api_client('/pulp/api/v3/docs/api.json')
    assert 'paths' in pulp_spec

    paths_keys = list(pulp_spec['paths'].keys())
    for ev in PULPY_VARIABLES:
        assert ev in paths_keys


@pytest.mark.standalone_only
@pytest.mark.openapi
@pytest.mark.openapi_generate_bindings
def test_openapi_bindings_generation(ansible_config):
    """Verify client bindings can be built from the pulp'ish api spec"""

    config = ansible_config("ansible_partner")
    api_client = get_client(
        config=config,
        request_token=True,
        require_auth=True
    )

    pulp_spec = api_client('/pulp/api/v3/docs/api.json')
    status = api_client('/pulp/api/v3/status/')
    version = [x['version'] for x in status['versions'] if x['component'] == 'galaxy'][0]
    my_id = subprocess.run(
        'id -u',
        shell=True,
        stdout=subprocess.PIPE,
        stderr=subprocess.PIPE
    ).stdout.decode('utf-8').strip()
    volume_name = '/local'
    generator_repo = 'https://github.com/pulp/pulp-openapi-generator'

    with tempfile.TemporaryDirectory(prefix='galaxy-bindings-') as output_dir:

        generator_checkout = os.path.join(output_dir, 'pulp-openapi-generator')
        clone_pid = subprocess.run(
            f'git clone {generator_repo} {generator_checkout}',
            shell=True,
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE
        )
        assert clone_pid.returncode == 0, clone_pid.stderr.decode('utf-8')

        with open(os.path.join(generator_checkout, 'api.json'), 'w') as f:
            f.write(json.dumps(pulp_spec))

        properties = '--additional-properties=packageName=pulpcore.client.galaxy_ng'
        properties += ',projectName=galaxy_ng-client'
        properties += f',packageVersion={version}'

        cmd = [
            'docker',
            'run',
            '-u',
            my_id,
            '--rm',
            '-v',
            f'{generator_checkout}:{volume_name}',
            'openapitools/openapi-generator-cli:v4.3.1',
            'generate',
            '-i', '/local/api.json',
            '-g', 'python',
            '-o', '/local/galaxy_ng-client',
            properties,
            '-t', '/local/templates/python',
            '--skip-validate-spec',
            '--strict-spec=false'
        ]

        docker_pid = subprocess.run(
            ' '.join(cmd),
            shell=True,
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE
        )
        assert docker_pid.returncode == 0, docker_pid.stderr.decode('utf-8')
        assert os.path.exists(os.path.join(generator_checkout, 'galaxy_ng-client'))



import logging
from pulpcore.plugin.tasking import dispatch
from pulp_ansible.app.tasks.signature import sign
from pulp_ansible.app.models import AnsibleRepository, CollectionVersionSignature

from .promotion import move_content

log = logging.getLogger(__name__)


def call_sign_and_move_task(signing_service, collection_version, source_repo, dest_repo):
    """Dispatches sign and move task

    This is a wrapper to group sign, copy_content and remove_content tasks
    because those 3 must run in sequence ensuring the same locks.
    """
    log.info(
        'Signing with `%s` and moving collection version `%s` from `%s` to `%s`',
        signing_service.name,
        collection_version.pk,
        source_repo.name,
        dest_repo.name
    )

    return dispatch(
        sign_and_move,
        exclusive_resources=[source_repo, dest_repo],
        kwargs=dict(
            signing_service_pk=signing_service.pk,
            collection_version_pk=collection_version.pk,
            source_repo_pk=source_repo.pk,
            dest_repo_pk=dest_repo.pk,
        )
    )


def sign_and_move(signing_service_pk, collection_version_pk, source_repo_pk, dest_repo_pk):
    """Sign collection version and then move to the destination repo"""

    # Sign while in the source repository
    sign(
        repository_href=source_repo_pk,
        content_hrefs=[collection_version_pk],
        signing_service_href=signing_service_pk
    )

    # Read signatures created on the source repository
    source_repo = AnsibleRepository.objects.get(pk=source_repo_pk)
    signatures_pks = CollectionVersionSignature.objects.filter(
        signed_collection=collection_version_pk,
        pk__in=source_repo.content.values_list("pk", flat=True)
    ).values_list("pk", flat=True)

    # Move content from source to destination
    move_content(
        collection_version_pk=collection_version_pk,
        source_repo_pk=source_repo_pk,
        dest_repo_pk=dest_repo_pk,
        signatures_pks=list(signatures_pks),
    )


def call_sign_task(signing_service, repository, content_units):
    """Calls task to sign collection content.
    signing_service: Instance of SigningService
    repository: Instance of AnsibleRepository
    content_units: List of content units UUIDS to sign or '*'
                   to sign all content units under repo
    """
    log.info(
        'Signing on-demand with `%s` on repository `%s` content `%s`',
        signing_service.name,
        repository.name,
        content_units
    )

    return dispatch(
        sign,
        exclusive_resources=[repository],
        kwargs=dict(
            repository_href=repository.pk,
            content_hrefs=content_units,
            signing_service_href=signing_service.pk
        )
    )



from django.conf import settings
from pulpcore.openapi import PulpSchemaGenerator


class GalaxySchemaGenerator(PulpSchemaGenerator):
    """Galaxy Schema Generator."""

    def convert_endpoint_path_params(self, path, view, schema):
        """Bypass variable-ization of paths if not a pulp route"""
        if hasattr(self, '_input_request'):
            if self._input_request.path.startswith(settings.GALAXY_API_PATH_PREFIX):
                return path
        return super().convert_endpoint_path_params(path, view, schema)

    def get_schema(self, request=None, public=False):
        """Munge pulp's get_schema result"""
        self._input_request = request
        schema = super().get_schema(request=request, public=public)
        if self._input_request.path.startswith(settings.GALAXY_API_PATH_PREFIX):
            return self.dedupe_operationIds(schema)
        return schema

    def dedupe_operationIds(self, schema):
        """Ensure all path+method operationIds are unique"""

        # group paths+methods by operationId
        operation_ids = {}
        for pk in schema['paths'].keys():
            for method, method_info in schema['paths'][pk].items():
                operationId = method_info['operationId']
                if operationId not in operation_ids:
                    operation_ids[operationId] = []
                operation_ids[operationId].append((pk, method))

        # make a new operationId for any duplicates
        for k, v in operation_ids.items():
            if len(v) < 2:
                continue
            for x in v:
                orig = schema['paths'][x[0]][x[1]]['operationId']
                schema['paths'][x[0]][x[1]]['operationId'] = \
                    self.flatten_path(x[0]) + '_' + orig

        return schema

    def flatten_path(self, path):
        """Assemble a new operationId prefix from a given path"""
        paths = [x for x in path.split('/')]
        return '_'.join(paths[:-1])



from django.core.management import CommandError, call_command
from django.test import TestCase
from pulp_ansible.app.models import AnsibleRepository


CMD = "set-retain-repo-versions"


class TestSetRetainRepoVersionsCommand(TestCase):

    @classmethod
    def setUpClass(cls):
        cls.original_data = list(
            AnsibleRepository.objects.all().values("name", "retain_repo_versions")
        )

    @classmethod
    def tearDownClass(cls):
        for data in cls.original_data:
            repo = AnsibleRepository.objects.get(name=data["name"])
            repo.retain_repo_versions = data["retain_repo_versions"]
            repo.save()

    def test_command_output(self):
        with self.assertRaisesMessage(
            CommandError, "Error: the following arguments are required: --value, --repository"
        ):
            call_command(CMD, "--yes")

    def test_set_all_to_2(self):
        """All the repos gets retain_repo_versions set to 2"""
        call_command(CMD, "--yes", "--value", "2", "--repository", "*")
        for repo in AnsibleRepository.objects.all():
            self.assertEqual(repo.retain_repo_versions, 2)

    def test_set_only_published_to_1000(self):
        """Only published is set to 1000"""
        repo = AnsibleRepository.objects.get(name="published")
        repo.retain_repo_versions = 1
        repo.save()
        call_command(CMD, "--yes", "--value", "1000", "--repository", "published")
        repo = AnsibleRepository.objects.get(name="published")

        self.assertEqual(repo.retain_repo_versions, 1000)

        for repo in AnsibleRepository.objects.exclude(name="published"):
            self.assertNotEqual(repo.retain_repo_versions, 1000)

    def test_set_only_published_and_stating_to_2000(self):
        """Only published,staging is set to 2000"""
        AnsibleRepository.objects.filter(name__in=["published", "staging"]).update(
            retain_repo_versions=1
        )
        call_command(CMD, "--yes", "--value", "2000", "--repository", "published,staging")

        for repo in AnsibleRepository.objects.filter(name__in=["published", "staging"]):
            self.assertEqual(repo.retain_repo_versions, 2000)

        for repo in AnsibleRepository.objects.exclude(name__in=["published", "staging"]):
            self.assertNotEqual(repo.retain_repo_versions, 2000)

    def test_set_all_to_10_excluding_published(self):
        """All the repos gets retain_repo_versions set to 10 except published"""
        call_command(CMD, "--yes", "--value", "10", "--repository", "*", "--exclude", "published")
        for repo in AnsibleRepository.objects.exclude(name="published"):
            self.assertEqual(repo.retain_repo_versions, 10)
        published = AnsibleRepository.objects.get(name="published")
        self.assertNotEqual(published.retain_repo_versions, 10)



from django.core.management.base import BaseCommand
from django.db import transaction
from pulp_ansible.app.models import AnsibleRepository


class Command(BaseCommand):
    """This command sets retain repo versions for repositories

    Example:

    django-admin set-retain-repo-versions --value=1 --repository=<repo_name>
    django-admin set-retain-repo-versions --value=1 --repository="*"
    django-admin set-retain-repo-versions --value=1 --repository="*" --exclude=<repo_name>
    """

    def echo(self, message):
        self.stdout.write(self.style.SUCCESS(message))

    def add_arguments(self, parser):
        parser.add_argument("--value", type=int, help="Value to set", required=True)
        parser.add_argument("--repository", type=str, help="Repository name", required=True)
        parser.add_argument(
            "--exclude", type=str, help="Repository name to exclude", required=False, default=None
        )
        parser.add_argument(
            "-y",
            "--yes",
            action="store_true",
            help="Skip confirmation",
            default=False,
            required=False,
        )

    def handle(self, *args, **options):

        repository = options["repository"].split(",")
        value = options["value"]
        exclude = options["exclude"].split(",") if options["exclude"] else []

        if "*" in repository:
            repositories = AnsibleRepository.objects.all()
        else:
            repositories = AnsibleRepository.objects.filter(name__in=repository)

        if exclude:
            repositories = repositories.exclude(name__in=exclude)

        repo_list = repositories.values_list("name", flat=True)

        if not options["yes"]:
            confirm = input(
                f"This will set retain_repo_versions to {value} for {len(repo_list)} repositories, "
                f"repo_list[:20]={repo_list[:20]}\n"
                "Proceed? (Y/n)"
            ).lower()
            while True:
                if confirm not in ("y", "n", "yes", "no"):
                    confirm = input('Please enter either "y/yes" or "n/no": ')
                    continue
                if confirm in ("y", "yes"):
                    break
                else:
                    self.echo("Process canceled.")
                    return

        with transaction.atomic():
            for repo in repositories:
                repo.retain_repo_versions = value
                repo.save()

        self.echo(
            f"Successfully set retain repo versions to {value} for {len(repo_list)} repositories"
        )



"""test_landing_page.py - Test related to landing page endpoint.
"""
import pytest
from ansible.galaxy.api import GalaxyError

from ..utils import get_client


def test_pulp_sync_enhancement_endpoints(ansible_config):
    """Tests whether the landing page returns the expected fields and numbers."""

    client = get_client(
        config=ansible_config("ansible_insights"), request_token=True, require_auth=True
    )

    # verify that the repo metadate endpoint works
    results = client("/api/automation-hub/v3/")
    assert "published" in results

    # verify that the unpaginated endpoints are disabled
    with pytest.raises(GalaxyError) as ctx:
        client("/api/automation-hub/v3/collections/all/", method="GET")
    assert ctx.value.http_code == 404

    with pytest.raises(GalaxyError) as ctx:
        client("/api/automation-hub/v3/collection_versions/all/", method="GET")
    assert ctx.value.http_code == 404

    # verify that the content/ prefix works correctly unpaginated endpoints are disabled
    with pytest.raises(GalaxyError) as ctx:
        client("/api/automation-hub/content/published/v3/collections/all/", method="GET")
    assert ctx.value.http_code == 404

    with pytest.raises(GalaxyError) as ctx:
        client("/api/automation-hub/content/published/v3/collection_versions/all/", method="GET")
    assert ctx.value.http_code == 404



from galaxy_ng.app.access_control import access_policy
from random import sample
from rest_framework.response import Response
from pulp_ansible.app.models import CollectionVersion, AnsibleDistribution
from galaxy_ng.app.models import Namespace
from galaxy_ng.app import settings
from galaxy_ng.app.api import base as api_base


class LandingPageView(api_base.APIView):
    permission_classes = [access_policy.LandingPageAccessPolicy]
    action = "retrieve"

    def get(self, request, *args, **kwargs):
        golden_name = settings.GALAXY_API_DEFAULT_DISTRIBUTION_BASE_PATH

        distro = AnsibleDistribution.objects.get(base_path=golden_name)
        repository_version = distro.repository.latest_version()
        collection_count = CollectionVersion.objects.filter(
            pk__in=repository_version.content, is_highest=True
        ).count()

        partner_count = Namespace.objects.count()

        # If there are no partners dont show the recommendation for it
        recommendations = {}
        if partner_count > 0:
            namespace = sample(list(Namespace.objects.all()), 1)[0]
            recommendations = {
                "recs": [
                    {
                        "id": "ansible-partner",
                        "icon": "bulb",
                        "action": {
                            "title": "Check out our partner %s" % namespace.company,
                            "href": "./ansible/automation-hub/partners/%s" % namespace.name,
                        },
                        "description": "Discover automation from our partners.",
                    }
                ]
            }

        data = {
            "estate": {
                "items": [
                    {
                        "id": "ansible-collections",
                        "count": collection_count,
                        "shape": {
                            "title": "Collections",
                            "href": "./ansible/automation-hub/",
                        },
                    },
                    {
                        "id": "ansible-partners",
                        "count": partner_count,
                        "shape": {
                            "title": "Partners",
                            "href": "./ansible/automation-hub/partners/",
                        },
                    },
                ],
            },
            "recommendations": recommendations,
            "configTryLearn": {
                "configure": [
                    {
                        "shape": {
                            "title": "Sync Red Hat certified collections",
                            "description": (
                                "Configure access to sync collections ",
                                "to Private Automation Hub.",
                            ),
                            "link": {
                                "title": "Get started",
                                "href": "./ansible/automation-hub/token",
                            },
                        },
                    }
                ],
                "try": [
                    {
                        "shape": {
                            "title": "Install Private Automation Hub",
                            "link": {
                                "title": "Get started",
                                "external": True,
                                "href": (
                                    "https://access.redhat.com/documentation/en-us/"
                                    "red_hat_ansible_automation_platform/2.1/html/"
                                    "red_hat_ansible_automation_platform_installation_guide/index"
                                ),
                            },
                        },
                    },
                    {
                        "shape": {
                            "title": "Manage repositories in Private Automation Hub",
                            "description": (
                                "Add community and privately developed collections "
                                "to your Private Automation Hub."
                            ),
                            "link": {
                                "title": "Get started",
                                "external": True,
                                "href": (
                                    "https://access.redhat.com/documentation/en-us/"
                                    "red_hat_ansible_automation_platform/2.1/html/"
                                    "publishing_proprietary_content_collections_in_"
                                    "automation_hub/index"
                                ),
                            },
                        },
                    },
                ],
                "learn": [
                    {
                        "shape": {
                            "title": "Connect Automation Hub to your automation infrastructure",
                            "link": {
                                "title": "Get started",
                                "external": True,
                                "href": (
                                    "https://docs.ansible.com/ansible-tower/latest/html/userguide/"
                                    "projects.html?extIdCarryOver=true&"
                                    "sc_cid=701f2000001Css5AAC#using-collections-in-tower"
                                ),
                            },
                        },
                    },
                    {
                        "shape": {
                            "title": "Learn about namespaces",
                            "description": (
                                "Organize collections content into " "namespaces users can access."
                            ),
                            "link": {
                                "title": "Learn more",
                                "external": True,
                                "href": (
                                    "https://access.redhat.com/documentation"
                                    "/en-us/red_hat_ansible_automation_platform/2.1/html"
                                    "/curating_collections_using_namespaces_in_automation_hub/index"
                                ),
                            },
                        },
                    },
                    {
                        "shape": {
                            "title": "Explore Red Hat certified collections",
                            "link": {
                                "title": "Learn more",
                                "external": True,
                                "href": "https://www.ansible.com/partners",
                            },
                        },
                    },
                ],
            },
        }

        return Response(data)



import logging

import requests
from django.conf import settings
from django.core.exceptions import ObjectDoesNotExist, ValidationError
from django.http import HttpResponseRedirect, StreamingHttpResponse
from django.shortcuts import get_object_or_404, redirect
from django.urls import reverse
from django.utils.translation import gettext_lazy as _
from drf_spectacular.utils import extend_schema
from pulp_ansible.app.galaxy.v3 import views as pulp_ansible_views
from pulp_ansible.app.models import AnsibleDistribution
from pulp_ansible.app.models import CollectionImport as PulpCollectionImport
from pulp_ansible.app.models import CollectionVersion

from pulpcore.plugin.models import Content
from pulpcore.plugin.models import SigningService
from pulpcore.plugin.models import Task
from pulpcore.plugin.serializers import AsyncOperationResponseSerializer
from pulpcore.plugin.tasking import dispatch
from rest_framework.exceptions import APIException, NotFound
from rest_framework.response import Response

from galaxy_ng.app import models
from galaxy_ng.app.access_control import access_policy
from galaxy_ng.app.api import base as api_base
from galaxy_ng.app.api.v3.serializers import CollectionUploadSerializer
from galaxy_ng.app.common import metrics
from galaxy_ng.app.common.parsers import AnsibleGalaxy29MultiPartParser
from galaxy_ng.app.constants import INBOUND_REPO_NAME_FORMAT, DeploymentMode
from galaxy_ng.app.tasks import (
    call_move_content_task,
    curate_all_synclist_repository,
    call_sign_and_move_task,
    import_and_auto_approve,
    import_and_move_to_staging,
)


log = logging.getLogger(__name__)


class CollectionUploadViewSet(api_base.LocalSettingsMixin,
                              pulp_ansible_views.CollectionUploadViewSet):
    permission_classes = [access_policy.CollectionAccessPolicy]
    parser_classes = [AnsibleGalaxy29MultiPartParser]
    serializer_class = CollectionUploadSerializer

    def _dispatch_import_collection_task(self, temp_file_pk, repository=None, **kwargs):
        """Dispatch a pulp task started on upload of collection version."""
        locks = []
        context = super().get_serializer_context()
        request = context.get("request", None)

        kwargs["temp_file_pk"] = temp_file_pk
        kwargs["username"] = request.user.username

        if repository:
            locks.append(repository)
            kwargs["repository_pk"] = repository.pk

        if settings.GALAXY_REQUIRE_CONTENT_APPROVAL:
            return dispatch(import_and_move_to_staging, exclusive_resources=locks, kwargs=kwargs)
        return dispatch(import_and_auto_approve, exclusive_resources=locks, kwargs=kwargs)

    # Wrap super().create() so we can create a galaxy_ng.app.models.CollectionImport based on the
    # the import task and the collection artifact details

    def _get_data(self, request):
        serializer = CollectionUploadSerializer(data=request.data, context={'request': request})
        serializer.is_valid(raise_exception=True)

        return serializer.validated_data

    @staticmethod
    def _get_path(kwargs, filename_ns):
        """Use path from '/content/<path>/v3/' or
           if user does not specify distribution base path
           then use an inbound distribution based on filename namespace.
        """

        # the legacy collection upload views don't get redirected and still have to use the
        # old path arg
        path = kwargs.get(
            'distro_base_path',
            kwargs.get('path', settings.ANSIBLE_DEFAULT_DISTRIBUTION_PATH)
        )

        if path == settings.ANSIBLE_DEFAULT_DISTRIBUTION_PATH:
            path = INBOUND_REPO_NAME_FORMAT.format(namespace_name=filename_ns)
        return path

    @staticmethod
    def _check_path_matches_expected_repo(path, filename_ns):
        """Reject if path does not match expected inbound format
           containing filename namespace.

        Examples:
        Reject if path is "staging".
        Reject if path does not start with "inbound-".
        Reject if path is "inbound-alice" but filename namepace is "bob".
        """

        distro = get_object_or_404(AnsibleDistribution, base_path=path)
        repo_name = distro.repository.name
        if INBOUND_REPO_NAME_FORMAT.format(namespace_name=filename_ns) == repo_name:
            return
        raise NotFound(
            _('Path does not match: "%s"')
            % INBOUND_REPO_NAME_FORMAT.format(namespace_name=filename_ns)
        )

    @extend_schema(
        description="Create an artifact and trigger an asynchronous task to create "
        "Collection content from it.",
        summary="Upload a collection",
        request=CollectionUploadSerializer,
        responses={202: AsyncOperationResponseSerializer},
    )
    def create(self, request, *args, **kwargs):
        data = self._get_data(request)
        filename = data['filename']

        path = self._get_path(kwargs, filename_ns=filename.namespace)

        try:
            namespace = models.Namespace.objects.get(name=filename.namespace)
        except models.Namespace.DoesNotExist:
            raise ValidationError(
                _('Namespace "{0}" does not exist.').format(filename.namespace)
            )

        self._check_path_matches_expected_repo(path, filename_ns=namespace.name)

        self.check_object_permissions(request, namespace)

        try:
            response = super(CollectionUploadViewSet, self).create(request, path)
        except ValidationError:
            log.exception('Failed to publish artifact %s (namespace=%s, sha256=%s)',  # noqa
                          data['file'].name, namespace, data.get('sha256'))
            raise

        task_href = response.data['task']

        # icky, have to extract task id from the task_href url
        task_id = task_href.strip("/").split("/")[-1]

        task_detail = Task.objects.get(pk=task_id)

        pulp_collection_import = PulpCollectionImport.objects.get(pk=task_id)

        models.CollectionImport.objects.create(
            task_id=pulp_collection_import,
            created_at=task_detail.pulp_created,
            namespace=namespace,
            name=data['filename'].name,
            version=data['filename'].version,
        )

        # TODO: CollectionImport.get_absolute_url() should be able to generate this, but
        #       it needs the  repo/distro base_path for the <path> part of url
        import_obj_url = reverse("galaxy:api:v3:collection-imports-detail",
                                 kwargs={'pk': str(task_detail.pk),
                                         'path': path})

        log.debug('import_obj_url: %s', import_obj_url)
        return Response(
            data={'task': import_obj_url},
            status=response.status_code
        )


class CollectionArtifactDownloadView(api_base.APIView):
    permission_classes = [access_policy.CollectionAccessPolicy]
    action = 'download'

    def _get_tcp_response(self, url):
        return requests.get(url, stream=True, allow_redirects=False)

    def _get_ansible_distribution(self, base_path):
        return AnsibleDistribution.objects.get(base_path=base_path)

    def get(self, request, *args, **kwargs):
        metrics.collection_artifact_download_attempts.inc()

        distro_base_path = self.kwargs['distro_base_path']
        filename = self.kwargs['filename']
        prefix = settings.CONTENT_PATH_PREFIX.strip('/')
        distribution = self._get_ansible_distribution(distro_base_path)

        if settings.GALAXY_DEPLOYMENT_MODE == DeploymentMode.INSIGHTS.value:
            url = 'http://{host}:{port}/{prefix}/{distro_base_path}/{filename}'.format(
                host=settings.X_PULP_CONTENT_HOST,
                port=settings.X_PULP_CONTENT_PORT,
                prefix=prefix,
                distro_base_path=distro_base_path,
                filename=filename,
            )
            response = self._get_tcp_response(
                distribution.content_guard.cast().preauthenticate_url(url)
            )

            if response.status_code == requests.codes.not_found:
                metrics.collection_artifact_download_failures.labels(
                    status=requests.codes.not_found
                ).inc()
                raise NotFound()
            if response.status_code == requests.codes.found:
                return HttpResponseRedirect(response.headers['Location'])
            if response.status_code == requests.codes.ok:
                metrics.collection_artifact_download_successes.inc()
                return StreamingHttpResponse(
                    response.raw.stream(amt=4096),
                    content_type=response.headers['Content-Type']
                )
            metrics.collection_artifact_download_failures.labels(status=response.status_code).inc()
            raise APIException(
                _('Unexpected response from content app. Code: %s.') % response.status_code
            )
        elif settings.GALAXY_DEPLOYMENT_MODE == DeploymentMode.STANDALONE.value:
            url = '{host}/{prefix}/{distro_base_path}/{filename}'.format(
                host=settings.CONTENT_ORIGIN.strip("/"),
                prefix=prefix,
                distro_base_path=distro_base_path,
                filename=filename,
            )
            return redirect(distribution.content_guard.cast().preauthenticate_url(url))


class CollectionVersionMoveViewSet(api_base.ViewSet):
    permission_classes = [access_policy.CollectionAccessPolicy]

    def move_content(self, request, *args, **kwargs):
        """Remove content from source repo and add to destination repo.

        Creates new RepositoryVersion of source repo without content included.
        Creates new RepositoryVersion of destination repo with content included.
        """

        version_str = '-'.join([self.kwargs[key] for key in ('namespace', 'name', 'version')])
        try:
            collection_version = CollectionVersion.objects.get(
                namespace=self.kwargs['namespace'],
                name=self.kwargs['name'],
                version=self.kwargs['version'],
            )
        except ObjectDoesNotExist:
            raise NotFound(_('Collection %s not found') % version_str)

        try:
            src_repo = AnsibleDistribution.objects.get(
                base_path=self.kwargs['source_path']).repository
            dest_repo = AnsibleDistribution.objects.get(
                base_path=self.kwargs['dest_path']).repository
        except ObjectDoesNotExist:
            raise NotFound(_('Repo(s) for moving collection %s not found') % version_str)

        content_obj = Content.objects.get(pk=collection_version.pk)

        if content_obj not in src_repo.latest_version().content:
            raise NotFound(_('Collection %s not found in source repo') % version_str)

        if content_obj in dest_repo.latest_version().content:
            raise NotFound(_('Collection %s already found in destination repo') % version_str)

        response_data = {
            "copy_task_id": None,
            "remove_task_id": None,
            "curate_all_synclist_repository_task_id": None,
        }
        golden_repo = settings.get("GALAXY_API_DEFAULT_DISTRIBUTION_BASE_PATH", "published")
        auto_sign = settings.get("GALAXY_AUTO_SIGN_COLLECTIONS", False)
        move_task_params = {
            "collection_version": collection_version,
            "source_repo": src_repo,
            "dest_repo": dest_repo,
        }

        if auto_sign and dest_repo.name == golden_repo:
            # Assumed that if user has access to modify the repo, they can also sign the content
            # so we don't need to check access policies here.
            signing_service_name = settings.get(
                "GALAXY_COLLECTION_SIGNING_SERVICE", "ansible-default"
            )
            try:
                signing_service = SigningService.objects.get(name=signing_service_name)
            except ObjectDoesNotExist:
                raise NotFound(_('Signing %s service not found') % signing_service_name)

            move_task = call_sign_and_move_task(signing_service, **move_task_params)
        else:
            move_task = call_move_content_task(**move_task_params)

        response_data['copy_task_id'] = response_data['remove_task_id'] = move_task.pk

        if settings.GALAXY_DEPLOYMENT_MODE == DeploymentMode.INSIGHTS.value:
            golden_repo = AnsibleDistribution.objects.get(
                base_path=settings.GALAXY_API_DEFAULT_DISTRIBUTION_BASE_PATH
            ).repository

            if dest_repo == golden_repo or src_repo == golden_repo:
                curate_task = dispatch(
                    curate_all_synclist_repository,
                    shared_resources=[golden_repo],
                    args=(golden_repo.name,),
                    kwargs={},
                )
                response_data['curate_all_synclist_repository_task_id'] = curate_task.pk

        return Response(data=response_data, status='202')



"""test_namespace_management.py - Test related to namespaces.

See: https://issues.redhat.com/browse/AAH-1303

"""
import pytest
import time

from ansible.galaxy.api import GalaxyError

from ..utils import get_client
from ..utils import get_all_collections_by_repo
from ..utils import get_all_repository_collection_versions
from ..utils import wait_for_task


pytestmark = pytest.mark.qa  # noqa: F821


@pytest.mark.galaxyapi_smoke
@pytest.mark.delete
@pytest.mark.collection_delete
def test_delete_collection(ansible_config, uncertifiedv2):
    """Tests whether a colleciton can be deleted"""

    config = ansible_config("ansible_partner")
    api_client = get_client(
        config=config,
        request_token=True,
        require_auth=True
    )

    # Enumerate published collection info ...
    collectionv1 = uncertifiedv2[0]
    cnamespace = collectionv1.namespace
    cname = collectionv1.name
    ckeys = []
    for cv in uncertifiedv2:
        ckeys.append((cv.namespace, cv.name, cv.version))

    # Try deleting the whole collection ...
    resp = api_client(
        ('/api/automation-hub/v3/plugin/ansible/content'
         f'/published/collections/index/{cnamespace}/{cname}/'),
        method='DELETE'
    )

    # wait for the orphan_cleanup job to finish ...
    try:
        wait_for_task(api_client, resp, timeout=10000)
    except GalaxyError as ge:
        # FIXME - pulp tasks do not seem to accept token auth
        if ge.http_code in [403, 404]:
            time.sleep(5)
        else:
            raise Exception(ge)

    # Make sure they're all gone ...
    after = get_all_collections_by_repo(api_client)
    for ckey in ckeys:
        assert ckey not in after['staging']
        assert ckey not in after['published']

    # Does the collection still exist?
    failed = None
    try:
        api_client(f'/api/automation-hub/collections/{cnamespace}/{cname}/')
        failed = False
    except GalaxyError as ge:
        if ge.http_code in [403, 404]:
            failed = True
        else:
            raise Exception(ge)

    assert failed


@pytest.mark.galaxyapi_smoke
@pytest.mark.delete
@pytest.mark.collection_version_delete
def test_delete_collection_version(ansible_config, upload_artifact, uncertifiedv2):
    """Tests whether a colleciton version can be deleted"""

    config = ansible_config("ansible_partner")
    api_client = get_client(
        config=config,
        request_token=True,
        require_auth=True
    )

    cv_before = get_all_repository_collection_versions(api_client)

    # Enumerate published collection info ...
    collectionv1 = uncertifiedv2[0]
    cnamespace = collectionv1.namespace
    cname = collectionv1.name

    # Delete each collection version from it's repo
    # specific href ...
    ckeys = []
    for cv in uncertifiedv2:
        ckey = (cv.namespace, cv.name, cv.version)
        ckeys.append(ckey)
        matches = []
        for k, v in cv_before.items():
            if k[1:] == ckey:
                matches.append(k)
        for rcv in matches:
            rcv_url = cv_before[rcv]['href']
            resp = api_client(rcv_url, method='DELETE')

            # wait for the orphan_cleanup job to finish ...
            try:
                wait_for_task(api_client, resp, timeout=10000)
            except GalaxyError as ge:
                # FIXME - pulp tasks do not seem to accept token auth
                if ge.http_code in [403, 404]:
                    time.sleep(5)
                else:
                    raise Exception(ge)

    # make sure the collection-versions are gone ...
    cv_after = get_all_repository_collection_versions(api_client)
    for ckey in ckeys:
        matches = []
        for k, v in cv_after.items():
            if k[1:] == ckey:
                matches.append(k)
        assert len(matches) == 0

    # make sure the collection was automatically purged
    # since all of it's children were deleted ...
    failed = None
    try:
        api_client(f'/api/automation-hub/collections/{cnamespace}/{cname}/')
        failed = False
    except GalaxyError as ge:
        if ge.http_code in [403, 404]:
            failed = True
        else:
            raise Exception(ge)

    assert failed

    assert failed



from galaxy_ng.app.api import base as api_base
from django.http import HttpResponseNotFound


class NotFoundView(api_base.APIView):
    permission_classes = []

    def get(self, request, *args, **kwargs):
        return HttpResponseNotFound()



"""test_cli_flow.py - Tests against the basic CLI publish/install behaviors."""
import logging
import pytest

from ..constants import USERNAME_PUBLISHER
from ..utils import ansible_galaxy
from ..utils import get_client
from ..utils import get_collection_full_path
from ..utils import create_unused_namespace
from ..utils import CollectionInspector
from ..utils import build_collection


pytestmark = pytest.mark.qa  # noqa: F821
logger = logging.getLogger(__name__)


@pytest.mark.cli
def test_publish_newer_version_collection(ansible_config, cleanup_collections, uncertifiedv2):
    """Test whether a newer version of collection can be installed after being published.

    If the collection version was not certified the version to be installed
    has to be specified during installation.
    """
    # FIXME - ^^^ is that really possible?

    v1 = uncertifiedv2[0]
    v2 = uncertifiedv2[1]

    # Install collection without version ...
    install_pid = ansible_galaxy(
        f"collection install {v1.namespace}.{v1.name}",
        ansible_config=ansible_config("ansible_partner"),
        cleanup=False,
        check_retcode=False
    )
    assert install_pid.returncode == 0, install_pid.stderr

    # Ensure the certified v1 version was installed ...
    collection_path = get_collection_full_path(v1.namespace, v1.name)
    ci = CollectionInspector(directory=collection_path)
    assert ci.version == v1.version
    assert ci.version != v2.version


@pytest.mark.cli
def test_publish_newer_certified_collection_version(
    ansible_config,
    cleanup_collections,
    certifiedv2
):
    """Test whether a newer certified collection version can be installed.

    If the collection version was certified the latest version will be installed.
    """

    v1 = certifiedv2[0]
    v2 = certifiedv2[1]

    # Ensure v2 gets installed by default ...
    ansible_galaxy(
        f"collection install {v1.namespace}.{v1.name}",
        ansible_config=ansible_config("ansible_partner")
    )
    collection_path = get_collection_full_path(v1.namespace, v1.name)
    ci = CollectionInspector(directory=collection_path)
    assert ci.version != v1.version
    assert ci.version == v2.version


@pytest.mark.cli
@pytest.mark.xfail
def test_publish_same_collection_version(ansible_config):
    """Test whether same collection version can be published."""

    api_client = get_client(ansible_config("ansible_insights"))
    cnamespace = create_unused_namespace(api_client=api_client)
    ansible_config("ansible_partner", namespace=USERNAME_PUBLISHER)
    collection = build_collection(namespace=cnamespace)
    ansible_galaxy(
        f"collection publish {collection.filename}",
        ansible_config=ansible_config("ansible_partner", namespace=collection.namespace)
    )
    p = ansible_galaxy(
        f"collection publish {collection.filename}",
        check_retcode=1,
        ansible_config=ansible_config("ansible_partner", namespace=collection.namespace)
    )
    assert "Artifact already exists" in str(p.stderr)


@pytest.mark.cli
def test_publish_and_install_by_self(ansible_config, published, cleanup_collections):
    """A publishing user has the permission to install an uncertified version of their
    own collection.
    """

    ansible_galaxy(
        f"collection install {published.namespace}.{published.name}:{published.version}",
        ansible_config=ansible_config("ansible_partner")
    )


@pytest.mark.cli
@pytest.mark.cloud_only
def test_publish_and_expect_uncertified_hidden(ansible_config, published, cleanup_collections):
    """A discovering/consumer user has the permission to download a specific version of an
    uncertified collection, but not an unspecified version range.
    """

    ansible_galaxy(
        f"collection install {published.namespace}.{published.name}", check_retcode=0,
        ansible_config=ansible_config("ansible_user")
    )
    ansible_galaxy(
        f"collection install {published.namespace}.{published.name}:1.0.0",
        ansible_config=ansible_config("ansible_user")
    )



"""test_dependencies.py - Tests of collection dependency handling."""
import logging

import attr
import pytest

from ..utils import ansible_galaxy
from ..utils import get_client
from ..utils import set_certification
from ..utils import build_collection

pytestmark = pytest.mark.qa  # noqa: F821

logger = logging.getLogger(__name__)


@attr.s
class DependencySpec:
    name = attr.ib()
    spec = attr.ib()
    retcode = attr.ib()
    xfail = attr.ib(default=False)


@pytest.mark.cli
@pytest.mark.parametrize(
    "params",
    (
        DependencySpec("normal", "1.0.0", 0),
        DependencySpec("exact", "=1.0.0", 0),
        DependencySpec("lt", "<2.0.0", 0),
        DependencySpec("lteq", "<=2.0.0", 0),
        DependencySpec("gt", ">0.9.0", 0),
        DependencySpec("gteq", ">=0.9.0", 0),
        DependencySpec("range", ">0.1.0,<1.0.1", 0),
        DependencySpec("invalid", "this is just junk", 1),
        # DependencySpec("carot", "^1.0.0", 1, xfail="galaxy-dev#104"),
        # DependencySpec("tilde", "~1.0.0", 1, xfail="galaxy-dev#104"),
        # DependencySpec("exception", ">0.0.0,!=1.0.0", 1, xfail="galaxy-dev#104"),
        # DependencySpec("missing1", "2.0.0", 1, xfail="galaxy-dev#104"),
        # DependencySpec("missing2", ">1.0.0", 1, xfail="galaxy-dev#104"),
    ),
    ids=lambda s: s.name,
)
def test_collection_dependency_install(ansible_config, published, cleanup_collections, params):
    """Collections defining dependencies can be installed and their dependencies are installed
    as well.

    Currently some scenarios are XFAIL pending open tickets:
    - Dependency specs with no matching collections (galaxy-dev#104)
    - NPM-style specs (not part of semver) are invalid
    """

    spec = params.spec
    retcode = params.retcode
    ansible_config("ansible_partner", namespace=published.namespace)
    artifact2 = build_collection(dependencies={f"{published.namespace}.{published.name}": spec})

    try:
        ansible_galaxy(
            f"collection publish {artifact2.filename} --server=automation_hub",
            check_retcode=retcode,
            ansible_config=ansible_config("ansible_partner", namespace=published.namespace)
        )
    except AssertionError:
        if params.xfail:
            return pytest.xfail()
        else:
            raise

    if retcode == 0:
        config = ansible_config("ansible_insights")
        client = get_client(config)
        set_certification(client, artifact2)

        pid = ansible_galaxy(
            f"collection install -vvv --ignore-cert \
                {artifact2.namespace}.{artifact2.name}:{artifact2.version} --server=automation_hub",
            check_retcode=False,
            ansible_config=ansible_config("ansible_insights"),
            # cleanup=False
        )

        try:
            assert (
                pid.returncode == retcode
            ), f"Unexpected {'failure' if pid.returncode else 'success'} during installing \
            {artifact2.namespace}.{artifact2.name}:{artifact2.version} \
            with dependency {published.namespace}.{published.name}{spec}"
        except AssertionError:
            raise
